/*******************************************************************************
 * Copyright (c) 2021, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <sys/types.h>

#pragma once

typedef unsigned short sa_family_t;
typedef unsigned short u_int16m_t; 
typedef unsigned int   u_int32m_t;

typedef unsigned long int in_addr_t;
typedef unsigned short in_port_t;

/* Address to accept any incoming messages.  */
#define INADDR_ANY  ((in_addr_t) 0x00000000)
/* Address to send to all hosts.  */
#define INADDR_BROADCAST ((in_addr_t) 0xffffffff)
/* Address indicating an error return.  */
#define INADDR_NONE ((in_addr_t) 0xffffffff)

/*ipv4*/
enum{
    IPPROTO_IP = 0,           /* Dummy protocol for TCP.  */
#define IPPROTO_IP                IPPROTO_IP
    IPPROTO_ICMP = 1,           /* Internet Control Message Protocol.  */
#define IPPROTO_ICMP                IPPROTO_ICMP
    IPPROTO_IGMP = 2,           /* Internet Group Management Protocol. */
#define IPPROTO_IGMP                IPPROTO_IGMP
    IPPROTO_IPIP = 4,           /* IPIP tunnels (older KA9Q tunnels use 94).  */
#define IPPROTO_IPIP                IPPROTO_IPIP
    IPPROTO_TCP = 6,           /* Transmission Control Protocol.  */
#define IPPROTO_TCP                IPPROTO_TCP
    IPPROTO_EGP = 8,           /* Exterior Gateway Protocol.  */
#define IPPROTO_EGP                IPPROTO_EGP
    IPPROTO_PUP = 12,           /* PUP protocol.  */
#define IPPROTO_PUP                IPPROTO_PUP
    IPPROTO_UDP = 17,           /* User Datagram Protocol.  */
#define IPPROTO_UDP                IPPROTO_UDP
    IPPROTO_IDP = 22,           /* XNS IDP protocol.  */
#define IPPROTO_IDP                IPPROTO_IDP
    IPPROTO_TP = 29,           /* SO Transport Protocol Class 4.  */
#define IPPROTO_TP                IPPROTO_TP
    IPPROTO_DCCP = 33,           /* Datagram Congestion Control Protocol.  */
#define IPPROTO_DCCP                IPPROTO_DCCP
    IPPROTO_IPV6 = 41,     /* IPv6 header.  */
#define IPPROTO_IPV6                IPPROTO_IPV6
    IPPROTO_RSVP = 46,           /* Reservation Protocol.  */
#define IPPROTO_RSVP                IPPROTO_RSVP
    IPPROTO_GRE = 47,           /* General Routing Encapsulation.  */
#define IPPROTO_GRE                IPPROTO_GRE
    IPPROTO_ESP = 50,      /* encapsulating security payload.  */
#define IPPROTO_ESP                IPPROTO_ESP
    IPPROTO_AH = 51,       /* authentication header.  */
#define IPPROTO_AH                IPPROTO_AH
    IPPROTO_MTP = 92,           /* Multicast Transport Protocol.  */
#define IPPROTO_MTP                IPPROTO_MTP
    IPPROTO_BEETPH = 94,   /* IP option pseudo header for BEET.  */
#define IPPROTO_BEETPH                IPPROTO_BEETPH
    IPPROTO_ENCAP = 98,           /* Encapsulation Header.  */
#define IPPROTO_ENCAP                IPPROTO_ENCAP
    IPPROTO_PIM = 103,           /* Protocol Independent Multicast.  */
#define IPPROTO_PIM                IPPROTO_PIM
    IPPROTO_COMP = 108,           /* Compression Header Protocol.  */
#define IPPROTO_COMP                IPPROTO_COMP
    IPPROTO_SCTP = 132,           /* Stream Control Transmission Protocol.  */
#define IPPROTO_SCTP                IPPROTO_SCTP
    IPPROTO_UDPLITE = 136, /* UDP-Lite protocol.  */
#define IPPROTO_UDPLITE                IPPROTO_UDPLITE
    IPPROTO_MPLS = 137,    /* MPLS in IP.  */
#define IPPROTO_MPLS                IPPROTO_MPLS
    IPPROTO_RAW = 255,           /* Raw IP packets.  */
#define IPPROTO_RAW                IPPROTO_RAW
    IPPROTO_MAX
};

/*ipv6*/
enum{
    IPPROTO_HOPOPTS = 0,   /* IPv6 Hop-by-Hop options.  */
#define IPPROTO_HOPOPTS                IPPROTO_HOPOPTS
    IPPROTO_ROUTING = 43,  /* IPv6 routing header.  */
#define IPPROTO_ROUTING                IPPROTO_ROUTING
    IPPROTO_FRAGMENT = 44, /* IPv6 fragmentation header.  */
#define IPPROTO_FRAGMENT        IPPROTO_FRAGMENT
    IPPROTO_ICMPV6 = 58,   /* ICMPv6.  */
#define IPPROTO_ICMPV6                IPPROTO_ICMPV6
    IPPROTO_NONE = 59,     /* IPv6 no next header.  */
#define IPPROTO_NONE                IPPROTO_NONE
    IPPROTO_DSTOPTS = 60,  /* IPv6 destination options.  */
#define IPPROTO_DSTOPTS                IPPROTO_DSTOPTS
    IPPROTO_MH = 135       /* IPv6 mobility header.  */
#define IPPROTO_MH                IPPROTO_MH
};

#ifdef __cplusplus
extern "C" {
#endif
    struct in_addr {
        unsigned long int s_addr;  // load with inet_aton()
    };

    struct in6_addr {
        unsigned char   s6_addr[16];
    };
    
    struct sockaddr_in {
        short            sin_family;   // e.g. AF_INET
        unsigned short   sin_port;     // e.g. htons(3490)
        struct in_addr   sin_addr;     // see struct in_addr, below
        char             sin_zero[8];  // zero this if you want to
    };

    struct sockaddr_in6 {
        unsigned char           sin6_len;      /* length of this structure */
        unsigned char           sin6_family;   /* AF_INET6                 */
        u_int16m_t              sin6_port;     /* Transport layer port #   */
        u_int32m_t              sin6_flowinfo; /* IPv6 flow information    */
        struct in6_addr         sin6_addr;     /* IPv6 address             */
    };
#ifdef __cplusplus
};
#endif
