/*******************************************************************************
 * Copyright (c) 2021, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <stddef.h>

#pragma once

#ifdef __cplusplus
extern "C" {
#endif
    
    typedef struct { int quot, rem; } div_t;
    typedef struct { long quot, rem; } ldiv_t;
    typedef struct { long long quot, rem; } lldiv_t;

    div_t div (int, int);
    ldiv_t ldiv (long, long);
    lldiv_t lldiv (long long, long long);

    void reverse(char s[]);
    void itoa(int n, char s[]);
    void ltoa(long n, char s[]);
    void ftoa(float n, char s[]);
    void ultoa(unsigned long n, char s[]);
    int atoi (const char* nptr);
    
    long long atoll(const char* nptr);
    unsigned long atoul(char* str);
    void ultoa(unsigned long n, char s[]);
    
    void exit (int status);
    
    void* calloc (size_t num, size_t size);
    void* malloc (size_t size);
    void free (void* ptr);
    void* realloc (void* ptr, size_t size);
    
    /**needs to be implemented**/
    int abs (int n);
    _Noreturn void abort();
    int atexit (void (*func)(void));
    int at_quick_exit (void (*func)(void));
    double atof (const char* str);
    long int atol ( const char * str );  
    void* bsearch (const void* key, const void* base,
                    size_t num, size_t size,
                    int (*compar)(const void*,const void*));
    int mblen (const char* pmb, size_t max);
    size_t mbstowcs (wchar_t* dest, const char* src, size_t max);
    int mbtowc (wchar_t* pwc, const char* pmb, size_t max);
    void qsort (void* base, size_t num, size_t size,
                int (*compar)(const void*,const void*));
    _Noreturn void quick_exit (int status);
    int rand (void);
    void srand (unsigned int seed);
    double strtod (const char* str, char** endptr);
    long double strtold (const char* str, char** endptr);
    long int strtol (const char* str, char** endptr, int base);
    long long int strtoll (const char* str, char** endptr, int base);
    unsigned long int strtoul (const char* str, char** endptr, int base);
    unsigned long long int strtoull (const char* str, char** endptr, int base);
    float strtof (const char* str, char** endptr);
    
    int system (const char* command);
    
    char* getenv (const char* name);
    
    _Noreturn void _Exit (int status);
    
    size_t wcstombs (char* dest, const wchar_t* src, size_t max);
    int wctomb (char* pmb, wchar_t wc);
    
    long int labs (long int n);
    long long int llabs (long long int n);
#ifdef __cplusplus
};
#endif
    
