/*******************************************************************************
 * Copyright (c) 2021, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <config.h>

#include <bits/alltypes.h>

#pragma once

#define MAP_FAILED              ((void *) -1)

#define MAP_GROWSDOWN           0x0100     /* stack-like segment */

#define MAP_SHARED              0x01       /* Share changes */
#define MAP_PRIVATE             0x02       /* Changes are private */
#define MAP_SHARED_VALIDATE     0x03

#define MAP_TYPE                0x0f       /* Mask for type of mapping */
#define MAP_FIXED               0x10       /* Interpret addr exactly */
#define MAP_ANONYMOUS           0x20       /* don't use a file */
#define MAP_ANON                0x20

#define PROT_READ               0x1        /* page can be read */
#define PROT_WRITE              0x2        /* page can be written */
#define PROT_EXEC               0x4        /* page can be executed */
#define PROT_SEM                0x8        /* page may be used for atomic ops */
/*			0x10                              reserved for arch-specific use */
/*			0x20	                          reserved for arch-specific use */
#define PROT_NONE               0x0        /* page can not be accessed */
#define PROT_GROWSDOWN          0x01000000 /* mprotect flag: extend change to start of growsdown vma */
#define PROT_GROWSUP            0x02000000 /* mprotect flag: extend change to end of growsup vma */

#define MAP_STACK               0x20000    /* Allocation is for a stack.  */

#define MAP_32BIT               0

#ifdef __cplusplus
extern "C" {
#endif

void *mmap (void *addr, size_t length, int prot, int flags, int fd, off_t offset);
int munmap (void *addr, size_t length);

int mprotect (void *addr, size_t len, int prot);

#ifdef __cplusplus
};
#endif
