/*******************************************************************************
 * Copyright (c) 2023, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <bits/signal.h>
#include <pthread.h>

#pragma once

#ifdef __cplusplus
extern "C" {
#endif
    union sigval {
        int sival_int;
        void *sival_ptr;
    };

    typedef struct {
        int si_signo;
        int si_code;
        union sigval si_value;
        int si_errno;
        pid_t si_pid;
        uid_t si_uid;
        void *si_addr;
        int si_status;
        int si_band;
    } siginfo_t;

    int __libc_current_sigrtmin(void) {
        return 0;
    }

    int __libc_current_sigrtmax(void) {
        return 0;
    }

    int kill(pid_t, int) {
        return 0;
    }

    int sigemptyset(sigset_t*) {
        return 0;
    }

    int sigfillset(sigset_t*) {
        return 0;
    }

    int sigaddset(sigset_t*, int) {
        return 0;
    }

    int sigdelset(sigset_t*, int) {
        return 0;
    }

    int sigismember(const sigset_t*, int) {
        return 0;
    }

    int sigprocmask(int, const sigset_t* __restrict__ sa, sigset_t* __restrict__ se) {
        return 0;
    }

    int sigsuspend(const sigset_t*);

    int sigaction(int sig, const struct sigaction* __restrict__ sa, struct sigaction* __restrict__ old);

    int sigpending(sigset_t*);

    int sigwait(const sigset_t* __restrict__ mask, int* __restrict__ sig);

    int sigwaitinfo(const sigset_t* __restrict, siginfo_t* __restrict) ;

    int sigtimedwait(const sigset_t* __restrict__ mask, siginfo_t* __restrict__ si, const struct timespec* __restrict__ timeout);

    int sigqueue(pid_t, int, const union sigval);

    int pthread_sigmask(int, const sigset_t* __restrict, sigset_t* __restrict);

    int pthread_kill(pthread_t, int);

    void psiginfo(const siginfo_t*, const char*);

    void psignal(int, const char*);

    int killpg(pid_t, int);

    int sigaltstack(const stack_t* __restrict, stack_t* __restrict);

    int sighold(int);

    int sigignore(int);

    int siginterrupt(int, int);

    int sigpause(int);

    int sigrelse(int);

    void (*sigset(int, void (*)(int)))(int);
    int raise(int sig);

    void (*signal(int, void (*)(int)))(int);
#ifdef __cplusplus
};
#endif
