/*******************************************************************************
 * Copyright (c) 2022, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/
#include <stddef.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/stat.h>

#pragma once

#define _SC_PIPE        145
#define _PC_PIPE_BUF    5


#define _SC_NPROCESSORS_CONF 1
#define _SC_NPROCESSORS_ONLN 2
#define _SC_PHYS_PAGES       3
#define _SC_AVPHYS_PAGES     4
#define _SC_PAGESIZE         6

#ifdef __cplusplus
extern "C" {
#endif

    typedef int pid_t;

    int pipe(int [2]);
    pid_t fork(void);

    int close(int fd);
    
    int chroot(const char *path);
    int chdir(const char *path);

    char *getcwd(char *buf, size_t size);
    char *getwd(char *buf);

    int stat(const char *path, struct stat *buf);
    int fstat(int fd, struct stat *buf);
    int lstat(const char *path, struct stat *buf);

    int setuid(uid_t uid);
    uid_t getuid(void);

    ssize_t write(int fd, const void *buf, size_t count);
    ssize_t read(int fd, void *buf, size_t count);

    int chmod(const char *pathname, mode_t mode);
    
    int execve(const char *pathname, char *const argv[],
                char *const envp[]);
    
    int getpagesize(void);

    ssize_t readlink(const char* path, char* buf, size_t bufsize);

    long sysconf(int name);

    int isatty(int fd);
#ifdef __cplusplus
}
#endif
