 /*******************************************************************************
 * Copyright (c) 2021, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <string.h>
#include <stdint.h>

#pragma once


namespace sys {
    template<class T>
    class array {
    public:
        void push_back(T t);
        void clear();
    };
    template<>
    class array<char> {
    public:
        array(){
            _buf=nullptr;
            _substr = nullptr;
            _size=0;
            _length = 0;
        };

	    array(array const &t){
            _buf=nullptr;
            _substr = nullptr;
            _size=0;
            _length = 0;
	        write(t.c_str(),t.size());
        };

        array(char *t){
            _buf = nullptr;
            _substr = nullptr;
            _size = 0;
            _length = 0;
            write(t,strlen(t));
	    }

        array(const char *t){
            _buf = nullptr;
            _substr = nullptr;
            _size = 0;
            _length = 0;
            write(t,strlen(t));
        }

        virtual ~array(){
             delete[] _buf;
        }

        array operator=(const char t) {
            clear();
            write(&t, sizeof(char));
            return *this;
        };

        void resize(size_t size) {
            char* tmp = _buf;
            _buf = new char[size];
            if (_length > size) {
                memcpy(_buf, tmp, size);
                _length = size;
            }
            else {
                memcpy(_buf, tmp, _length);
            }
            _size = size;
            delete[] tmp;
        }

        void write(const char *t,unsigned long size){
            if (int(_size - _length) <= size) {
                resize((((_length * 1.5) +1 )+ size));
            }
            memcpy(_buf + _length, t, size);
            _length += size;
            _buf[_length] = '\0';
        }

        array operator=(const char *t){
            clear();
            if(t)
                write(t,strlen(t));
            return *this;
        }

        array operator=(const array &t){
            clear();
	        write(t._buf,t._length);
	        return *this;
	    }

        array operator+=(const char *t){
            if(t)
                write(t,strlen(t));
            return *this;
        }

        array operator+=(const array &t){
                write(t._buf,t._length);
                return *this;
        }


        bool operator==(const char *t){
            if(!_buf && !t)
                return true;
            if(!_buf || !t)
                return false;
            if(_length!=strlen(t))
                return false;
            if(strncmp(_buf,t,_length)==0)
                return true;
            return false;
        }

        bool operator==(array t) {
            return this->operator==(t.c_str());
        }

        bool operator!=(const char* t) {
            return (!this->operator==(t));
        }

        bool operator!=(array t) {
            return this->operator!=(t.c_str());
        }

        char& operator[](int pos){
            return _buf[pos];
        };

        size_t size() const{
            return _size;
        };

        size_t length() const {
            return _length;
        };

        void push_back(char t){
            write(&t,sizeof(char));
        };

        void append(const char *src){
            write(src,strlen(src));
        }

        void clear(){
            delete[] _buf;
            _size=0;
            _length = 0;
            _buf=nullptr;
        };

        constexpr const char *c_str() const noexcept{
            return _buf;
        };

        bool empty(){
            if(_length==0)
                return true;
            return false;
        }

        array substr (size_t pos = 0, size_t len = -1) const{
            if (_substr)
                delete _substr;
            if (len == -1)
                len = _length;
            char *tmp=new char[len+1];
	        memcpy(tmp,_buf+pos,len);
            tmp[len] = '\0';
            array<char> _substr(tmp);
            delete[] tmp;
            return _substr;
        }

        array(array&&) = default;

    private:
        array<char>   *_substr;
        char          *_buf;
        size_t         _size;
        size_t         _length;
    };
};
