/*******************************************************************************
Copyright (c) 2021, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <bits/sysbits.h>
#include <bits/syscall.h>

#include "exception.h"
#include "console.h"

sys::__internal::_consoleout::_consoleout(int fd){
    _FD=fd;
}

sys::__internal::_consoleout::~_consoleout(){
}

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<< (const char* out){
    if(!out)
        return *this;
    write(_FD,(void*)out,strlen(out));
    return *this;    
}

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<< (int out){
    char buf[255];
    snprintf(buf,255,"%d",out);
    write(_FD,(void*)buf,strlen(buf));
    return *this;
}

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<< (long out){
    char buf[512];
    snprintf(buf,512,"%ld",out);
    write(_FD,(void*)buf,strlen(buf));
    return *this;
}

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<< (unsigned long out){
    char buf[255];
    snprintf(buf,255,"%lu",out);
    write(_FD,buf,strlen(buf));
    return *this;
}

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<< (unsigned int out){
    char buf[255];
    snprintf(buf,255,"%u",out);
    write(_FD,buf,strlen(buf));
    return *this;
}

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<< (sys::array<char> out){
    const char *buf=out.c_str();
    write(_FD,(void*)buf,strlen(buf));
    return *this;
}

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<<(double out){
    char buf[255];
    snprintf(buf,255,"%f",out);
    write(_FD,buf,strlen(buf));
    return *this;
};

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<<(float out){
    char buf[255];
    snprintf(buf,255,"%f",out);
    write(_FD,buf,strlen(buf));
    return *this;
};

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<<(long double out){
    char buf[255];
    snprintf(buf,255,"%Lf",out);
    write(_FD,buf,strlen(buf));
    return *this;
};

sys::__internal::_consoleout &sys::__internal::_consoleout::operator<< (char out){
    write(_FD,&out,sizeof(char));
    return *this;
}

sys::__internal::_consoleout & sys::__internal::_consoleout::operator<<(sys::__internal::_consoleout& console){
    return console;
}

sys::__internal::_consolein::_consolein(int fd){
    _FD=fd;
}

sys::__internal::_consolein::~_consolein(){
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>> (int &in){
    char buf,tmp[255];
    int f=0,i=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        tmp[i++]=buf;
    }while(true);
    tmp[i]='\0';
    snprintf(tmp,255,"%d",in);
    return *this;
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>> (unsigned long &in){
    char buf,tmp[255];
    int f=0,i=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        tmp[i++]=buf;
    }while(true);
    tmp[i]='\0';
    snprintf(tmp,255,"%lu",in);                                   
    return *this;
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>> (unsigned int &in){
    char buf,tmp[255];
    int f=0,i=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        tmp[i++]=buf;
    }while(true);
    tmp[i]='\0';
    snprintf(tmp,255,"%u",in);
    return *this;
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>>(double &in){
    char buf,tmp[255];
    int f=0,i=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        tmp[i++]=buf;
    }while(true);
    tmp[i]='\0';
    snprintf(tmp,255,"%f",in);
    return *this;
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>>(float &in){
    char buf,tmp[255];
    int f=0,i=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        tmp[i++]=buf;
    }while(true);
    tmp[i]='\0';
    snprintf(tmp,255,"%f",in);
    return *this;
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>>(long double &in){
    char buf,tmp[255];
    int f=0,i=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        tmp[i++]=buf;
    }while(true);
    tmp[i]='\0';
    snprintf(tmp,255,"%Lf",in);
    return *this;
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>> (char &in){
    char buf,tmp[255];
    int f=0,i=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        tmp[i++]=buf;
    }while(true);
    tmp[i]='\0';
    char *str=new char[i];
    memcpy(str,tmp,i);
    in=*str;
    return *this;
}

sys::__internal::_consolein &sys::__internal::_consolein::operator>> (sys::array<char> &in){
    char buf;
    int f=0;
    do{
        f=read(_FD,&buf,1);
        if(buf=='\n')
            break;
        in.push_back(buf);
    }while(true);
    return *this;
}

sys::__internal::_consolein & sys::__internal::_consolein::operator>>(sys::__internal::_consolein& console){
    return console;
}
