/*******************************************************************************
Copyright (c) 2022, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
	* Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	* Neither the name of the <organization> nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>

#include <sys/mman.h>

#include "cxa_exception.h"

#define _ATEXIT_LOCK() _thread_atexit_lock()
#define _ATEXIT_UNLOCK() _thread_atexit_unlock()

static pthread_mutex_t gAtExitLock = PTHREAD_MUTEX_INITIALIZER;

extern "C" {    
    
    void  _thread_atexit_lock( void ){
        pthread_mutex_lock( &gAtExitLock );
    }

    void _thread_atexit_unlock( void ){
        pthread_mutex_unlock( &gAtExitLock );
    }

    void _thread_finalize(void){
        //needs rewrite
//             struct tib *tib = TIB_GET();
//             
//             while (tib->tib_atexit) {
//                 struct thread_atexit_fn *fnp = tib->tib_atexit;
//                 tib->tib_atexit = fnp->next;
//                 fnp->func(fnp->arg);
//                 ::sys::allocator::getInstance().free(fnp);
//             }
    }
    
    int __cxa_atexit(void (*f)(void *), void *objptr, void *dso){
        struct atexit *p = __atexit;
        struct atexit::atexit_fn *fnp;
        int pgsize = getpagesize();
        int ret = -1;
        
        if (pgsize < sizeof(*p))
            return (-1);
        _ATEXIT_LOCK();
        p = __atexit;
        if (p != NULL) {
            if (p->ind + 1 >= p->max)
                p = NULL;
            else if (mprotect(p, pgsize, PROT_READ | PROT_WRITE))
                goto unlock;
        }
        if (p == NULL) {
            p =(struct atexit*) mmap(NULL, pgsize, PROT_READ | PROT_WRITE,
                     MAP_ANON | MAP_PRIVATE, -1, 0);
            if (p == MAP_FAILED)
                goto unlock;
            if (__atexit == NULL) {
                memset(&p->fns[0], 0, sizeof(p->fns[0]));
                p->ind = 1;
            } else
                p->ind = 0;
            p->max = (pgsize - ((char *)&p->fns[0] - (char *)p)) /
            sizeof(p->fns[0]);
            p->next = __atexit;
            __atexit = p;
        }
        fnp = &p->fns[p->ind++];
        fnp->fn_ptr = f;
        fnp->fn_arg = objptr;
        fnp->fn_dso = dso;
        if (mprotect(p, pgsize, PROT_READ))
            goto unlock;
        restartloop = 1;
        ret = 0;
        unlock:
        _ATEXIT_UNLOCK();
        return (ret);
    };
       
    void __cxa_finalize(void *f){
        struct atexit *p, *q;
        struct atexit::atexit_fn fn;
        int n, pgsize = getpagesize();
        static int call_depth;
        
        if (f == NULL)
            _thread_finalize();
        
        _ATEXIT_LOCK();
        call_depth++;
        
        restart:
        restartloop = 0;
        for (p = __atexit; p != NULL; p = p->next) {
            for (n = p->ind; --n >= 0;) {
                if (p->fns[n].fn_ptr == NULL)
                    continue;	/* already called */
                if (f != NULL && f != p->fns[n].fn_dso)
                    continue;	/* wrong DSO */
                        
                /*
                * Mark handler as having been already called to avoid
                * dupes and loops, then call the appropriate function.
                */
                fn = p->fns[n];
                if (mprotect(p, pgsize, PROT_READ | PROT_WRITE) == 0) {
                    p->fns[n].fn_ptr = NULL;
                    mprotect(p, pgsize, PROT_READ);
                }
                _ATEXIT_UNLOCK();
                (*fn.fn_ptr)(fn.fn_arg);
                _ATEXIT_LOCK();
                if (restartloop)
                    goto restart;
            }
        }
        
        call_depth--;
        
        /*
         * If called via exit(), unmap the pages since we have now run
         * all the handlers.  We defer this until calldepth == 0 so that
         * we don't unmap things prematurely if called recursively.
         */
        if (f == NULL && call_depth == 0) {
            for (p = __atexit; p != NULL; ) {
                q = p;
                p = p->next;
                munmap(q, pgsize);
            }
            __atexit = NULL;
        }
        _ATEXIT_UNLOCK();
        
        /*
         * If unloading a DSO, unregister any atfork handlers registered
         * by it.  Skip the locking if the list is currently empty.
         */
        //needs rewrite
//         if (f != NULL && TAILQ_FIRST(&_atfork_list) != NULL) {
//             struct atfork_fn *af, *afnext;
//             
//             _ATFORK_LOCK();
//             TAILQ_FOREACH_SAFE(af, &_atfork_list, fn_next, afnext)
//             if (af->fn_dso == f) {
//                 TAILQ_REMOVE(&_atfork_list, af, fn_next);
//                 free(af);
//             }
//             _ATFORK_UNLOCK();
//             
//         }
    };
    
    void __cxa_call_unexpected(void *u){
        
    };
    
    void __cxa_begin_catch(void *c){
    };
    
    void __cxa_end_catch() {
        
    }
      
    void __cxa_pure_virtual(){
        
    };
    
    void __cxa_throw (void *thrown_exception, std::type_info *tinfo, void (*dest) (void *) ){
        exit(1);
    }
    
    void *__cxa_allocate_exception(unsigned long thrown_size){
        return operator new(thrown_size);
    };
    
    void __cxa_free_exception(void *e){
        operator delete(e);;
    };
    
    __extension__ typedef int __guard __attribute__((mode(__DI__)));
     
    int __cxa_guard_acquire (__guard *g) {
        return 0;
    }
 
    void __cxa_guard_release (__guard *g){
    }
 
    void __cxa_guard_abort (__guard *){
 
    }
    
    void __attribute__((weak)) __cxa_throw_bad_array_new_length() {
    };


};

