/*******************************************************************************
Copyright (c) 2022, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
	* Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	* Neither the name of the <organization> nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#pragma once

#define ATEXIT_MAX_FUNCS	128

extern "C" {

typedef unsigned uarch_t;
typedef void*    unexpected_handler;
typedef void*    terminate_handler;

struct atexit {
    struct atexit *next;		/* next in list */
    int ind;			/* next index in this table */
    int max;			/* max entries >= ATEXIT_SIZE */
    struct atexit_fn {
        void (*fn_ptr)(void *);
        void *fn_arg;		/* argument for CXA callback */
        void *fn_dso;		/* shared module handle */
    } fns[1];			/* the table itself */
};

/* a chain of these are hung off each thread's TIB's tib_atexit member */
struct thread_atexit_fn {
    void (*func)(void *);
    void *arg;
    struct thread_atexit_fn *next;
};

struct atexit *__atexit;	

static int restartloop;

namespace std {
    class type_info; // forward declaration
}
    
struct __cxa_exception {
    std::type_info      *exceptionType;
    void                 (*exceptionDestructor) (void *); 
    unexpected_handler   unexpectedHandler;
    terminate_handler    terminateHandler;
    __cxa_exception     *nextException;
    
    int                  handlerCount;
    int                  handlerSwitchValue;
    const char          *actionRecord;
    const char          *languageSpecificData;
    void                *catchTemp;
    void                *adjustedPtr;
};

int __cxa_atexit(void (*f)(void *), void *objptr, void *dso);
void __cxa_finalize(void *f);
};
