/*******************************************************************************
Copyright (c) 2022, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#pragma once

/* Types of sockets.  */
enum __socket_type
{
    SOCK_STREAM = 1,                /* Sequenced, reliable, connection-based
                                     byte streams.  */
#define SOCK_STREAM SOCK_STREAM
    SOCK_DGRAM = 2,                /* Connectionless, unreliable datagrams
                                     of fixed maximum length.  */
#define SOCK_DGRAM SOCK_DGRAM
    SOCK_RAW = 3,                        /* Raw protocol interface.  */
#define SOCK_RAW SOCK_RAW
    SOCK_RDM = 4,                        /* Reliably-delivered messages.  */
#define SOCK_RDM SOCK_RDM
    SOCK_SEQPACKET = 5,                /* Sequenced, reliable, connection-based,
                                     datagrams of fixed maximum length.  */
#define SOCK_SEQPACKET SOCK_SEQPACKET
    SOCK_DCCP = 6,                /* Datagram Congestion Control Protocol.  */
#define SOCK_DCCP SOCK_DCCP
    SOCK_PACKET = 10,                /* Linux specific way of getting packets
                                     at the dev level.  For writing rarp and
                                     other similar things on the user level. */
#define SOCK_PACKET SOCK_PACKET

                                     /* Flags to be ORed into the type parameter of socket and socketpair and
                                        used for the flags parameter of paccept.  */

    SOCK_CLOEXEC = 02000000,        /* Atomically set close-on-exec flag for the
                                     new descriptor(s).  */
#define SOCK_CLOEXEC SOCK_CLOEXEC
    SOCK_NONBLOCK = 04000                /* Atomically mark descriptor(s) as
                                     non-blocking.  */
#define SOCK_NONBLOCK SOCK_NONBLOCK
};

#define SOL_SOCKET        1

#define SO_DEBUG          1
#define SO_REUSEADDR      2
#define SO_TYPE           3
#define SO_ERROR          4
#define SO_DONTROUTE      5
#define SO_BROADCAST      6
#define SO_SNDBUF         7
#define SO_RCVBUF         8
#define SO_SNDBUFFORCE    32
#define SO_RCVBUFFORCE    33
#define SO_KEEPALIVE      9
#define SO_OOBINLINE      10
#define SO_NO_CHECK       11
#define SO_PRIORITY       12
#define SO_LINGER         13
#define SO_BSDCOMPAT      14

/* Protocol families.  */
#define PF_UNSPEC         0        /* Unspecified.  */
#define PF_LOCAL          1        /* Local to host (pipes and file-domain).  */
#define PF_UNIX           PF_LOCAL /* POSIX name for PF_LOCAL.  */
#define PF_FILE           PF_LOCAL /* Another non-standard name for PF_LOCAL.  */
#define PF_INET           2        /* IP protocol family.  */
#define PF_AX25           3        /* Amateur Radio AX.25.  */
#define PF_IPX            4        /* Novell Internet Protocol.  */
#define PF_APPLETAL       5        /* Appletalk DDP.  */
#define PF_NETROM         6        /* Amateur radio NetROM.  */
#define PF_BRIDGE         7        /* Multiprotocol bridge.  */
#define PF_ATMPVC         8        /* ATM PVCs.  */
#define PF_X25            9        /* Reserved for X.25 project.  */
#define PF_INET6          10       /* IP version 6.  */
#define PF_ROSE           11       /* Amateur Radio X.25 PLP.  */
#define PF_DECnet         12       /* Reserved for DECnet project.  */
#define PF_NETBEUI        13       /* Reserved for 802.2LLC project.  */
#define PF_SECURITY       14       /* Security callback pseudo AF.  */
#define PF_KEY            15       /* PF_KEY key management API.  */
#define PF_NETLINK        16
#define PF_ROUTE          PF_NETLINK /* Alias to emulate 4.4BSD.  */
#define PF_PACKET         17       /* Packet family.  */
#define PF_ASH            18       /* Ash.  */
#define PF_ECONET         19       /* Acorn Econet.  */
#define PF_ATMSVC         20       /* ATM SVCs.  */
#define PF_RDS            21       /* RDS sockets.  */
#define PF_SNA            22       /* Linux SNA Project */
#define PF_IRDA           23       /* IRDA sockets.  */
#define PF_PPPOX          24       /* PPPoX sockets.  */
#define PF_WANPIPE        25       /* Wanpipe API sockets.  */
#define PF_LLC            26       /* Linux LLC.  */
#define PF_CAN            29       /* Controller Area Network.  */
#define PF_TIPC           30       /* TIPC sockets.  */
#define PF_BLUETOOTH      31       /* Bluetooth sockets.  */
#define PF_IUCV           32       /* IUCV sockets.  */
#define PF_RXRPC          33       /* RxRPC sockets.  */
#define PF_ISDN           34       /* mISDN sockets.  */
#define PF_PHONET         35       /* Phonet sockets.  */
#define PF_IEEE802154     36       /* IEEE 802.15.4 sockets.  */
#define PF_MAX            37       /* For now..  */

/* Address families.  */
#define AF_UNSPEC         PF_UNSPEC
#define AF_LOCAL          PF_LOCAL
#define AF_UNIX           PF_UNIX
#define AF_FILE           PF_FILE
#define AF_INET           PF_INET
#define AF_AX25           PF_AX25
#define AF_IPX            PF_IPX
#define AF_APPLETALK      PF_APPLETALK
#define AF_NETROM         PF_NETROM
#define AF_BRIDGE         PF_BRIDGE
#define AF_ATMPVC         PF_ATMPVC
#define AF_X25            PF_X25
#define AF_INET6          PF_INET6
#define AF_ROSE           PF_ROSE
#define AF_DECnet         PF_DECnet
#define AF_NETBEUI        PF_NETBEUI
#define AF_SECURITY       PF_SECURITY
#define AF_KEY            PF_KEY
#define AF_NETLINK        PF_NETLINK
#define AF_ROUTE          PF_ROUTE
#define AF_PACKET         PF_PACKET
#define AF_ASH            PF_ASH
#define AF_ECONET         PF_ECONET
#define AF_ATMSVC         PF_ATMSVC
#define AF_RDS            PF_RDS
#define AF_SNA            PF_SNA
#define AF_IRDA           PF_IRDA
#define AF_PPPOX          PF_PPPOX
#define AF_WANPIPE        PF_WANPIPE
#define AF_LLC            PF_LLC
#define AF_CAN            PF_CAN
#define AF_TIPC           PF_TIPC
#define AF_BLUETOOTH      PF_BLUETOOTH
#define AF_IUCV           PF_IUCV
#define AF_RXRPC          PF_RXRPC
#define AF_ISDN           PF_ISDN
#define AF_PHONET         PF_PHONET
#define AF_IEEE802154     PF_IEEE802154
#define AF_MAX            PF_MAX

/* Socket level values.  Others are defined in the appropriate headers.
   XXX These definitions also should go into the appropriate headers as
   far as they are available.  */
#define SOL_RAW          255
#define SOL_DECNET       261
#define SOL_X25          262
#define SOL_PACKET       263
#define SOL_ATM          264        /* ATM layer (cell level).  */
#define SOL_AAL          265        /* ATM Adaption Layer (packet level).  */
#define SOL_IRDA         266

/* Maximum queue length specifiable by listen.  */
#define SOMAXCONN        128

struct sockaddr {
    sa_family_t   sa_family; //address family
    char          sa_data[]; //socket address(variable - length data)
};