/*******************************************************************************
Copyright (c) 2022, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <config.h>
#include <stddef.h>
#include <stdio.h>

// #include <systempp/syself.h>
// #include <reloc.h>
//
// #define SHARED
//
// #ifndef START
// #define START "_dlstart"
// #endif
//
// #define AT_NULL		0
// #define AT_IGNORE	1
// #define AT_EXECFD	2
// #define AT_PHDR		3
// #define AT_PHENT	4
// #define AT_PHNUM	5
// #define AT_PAGESZ	6
// #define AT_BASE		7
// #define AT_FLAGS	8
// #define AT_ENTRY	9
// #define AT_NOTELF	10
// #define AT_UID		11
// #define AT_EUID		12
// #define AT_GID		13
// #define AT_EGID		14
// #define AT_CLKTCK	17
//
//
// #define AT_PLATFORM	15
// #define AT_HWCAP	16
//
// #define AUX_CNT 32
// #define DYN_CNT 32
//
// #define DT_RELRSZ	35
// #define DT_RELR		36
// #define DT_RELRENT	37
// #define	DT_NUM		38
//
// #define R_TYPE(x) ((x)&0x7fffffff)
// #define R_SYM(x) ((x)>>32)
//
// enum {
// 	REL_NONE = 0,
// 	REL_SYMBOLIC = -100,
// 	REL_GOT,
// 	REL_PLT,
// 	REL_RELATIVE,
// 	REL_OFFSET,
// 	REL_OFFSET32,
// 	REL_COPY,
// 	REL_SYM_OR_REL,
// 	REL_DTPMOD,
// 	REL_DTPOFF,
// 	REL_TPOFF,
// 	REL_TPOFF_NEG,
// 	REL_TLSDESC,
// 	REL_FUNCDESC,
// 	REL_FUNCDESC_VAL,
// };
//
// #if !DL_FDPIC
// #define IS_RELATIVE(x,s) ( \
// 	(R_TYPE(x) == REL_RELATIVE) || \
// 	(R_TYPE(x) == REL_SYM_OR_REL && !R_SYM(x)) )
// #else
// #define IS_RELATIVE(x,s) ( ( \
// 	(R_TYPE(x) == REL_FUNCDESC_VAL) || \
// 	(R_TYPE(x) == REL_SYMBOLIC) ) \
// 	&& (((s)[R_SYM(x)].st_info & 0xf) == STT_SECTION) )
// #endif
//
// __asm__(
// 	".text \n"
// 	".global " START " \n"
// 	START ": \n"
// 	"	xor %rbp,%rbp \n"
// 	"	mov %rsp,%rdi \n"
// 	".weak _DYNAMIC \n"
// 	".hidden _DYNAMIC \n"
// 	"	lea _DYNAMIC(%rip),%rsi \n"
// 	"	andq $-16,%rsp \n"
// 	"	call " START "_c \n"
// );
//
// typedef void (*stage2_func)(unsigned char*, size_t*);
//
// #ifndef GETFUNCSYM
// #define GETFUNCSYM(fp, sym, got) do { \
// 	hidden void sym(); \
// 	static void (*static_func_ptr)() = sym; \
// 	__asm__ __volatile__ ( "" : "+m"(static_func_ptr) : : "memory"); \
// 	*(fp) = static_func_ptr; } while(0)
// #endif
//
// typedef struct {
// 	Elf64_Word	p_type;
// 	Elf64_Word	p_flags;
// 	Elf64_Off	p_offset;
// 	Elf64_Addr	p_vaddr;
// 	Elf64_Addr	p_paddr;
// 	Elf64_Xword	p_filesz;
// 	Elf64_Xword	p_memsz;
// 	Elf64_Xword	p_align;
// } Phdr;
//
// extern "C" {
// 	__attribute__((visibility("hidden"))) void _dlstart_c(size_t* sp, size_t* dynv) {
// 		size_t i, aux[AUX_CNT], dyn[DYN_CNT];
// 		size_t* rel, rel_size, base;
//
// 		int argc = *sp;
// 		char** argv = (char**)(void*)(sp + 1);
//
// 		for (i = argc + 1; argv[i]; i++);
// 		size_t* auxv = (size_t*)(void*)(argv + i + 1);
//
// 		for (i = 0; i < AUX_CNT; i++) aux[i] = 0;
// 		for (i = 0; auxv[i]; i += 2) if (auxv[i] < AUX_CNT)
// 			aux[auxv[i]] = auxv[i + 1];
//
// #if DL_FDPIC
// 		struct fdpic_loadseg* segs, fakeseg;
// 		size_t j;
// 		if (dynv) {
// 			/* crt_arch.h entry point asm is responsible for reserving
// 			 * space and moving the extra fdpic arguments to the stack
// 			 * vector where they are easily accessible from C. */
// 			segs = ((struct fdpic_loadmap*)(sp[-1] ? sp[-1] : sp[-2]))->segs;
// 		}
// 		else {
// 			/* If dynv is null, the entry point was started from loader
// 			 * that is not fdpic-aware. We can assume normal fixed-
// 			 * displacement ELF loading was performed, but when ldso was
// 			 * run as a command, finding the Ehdr is a heursitic: we
// 			 * have to assume Phdrs start in the first 4k of the file. */
// 			base = aux[AT_BASE];
// 			if (!base) base = aux[AT_PHDR] & -4096;
// 			segs = &fakeseg;
// 			segs[0].addr = base;
// 			segs[0].p_vaddr = 0;
// 			segs[0].p_memsz = -1;
// 			Ehdr* eh = (void*)base;
// 			Phdr* ph = (void*)(base + eh->e_phoff);
// 			size_t phnum = eh->e_phnum;
// 			size_t phent = eh->e_phentsize;
// 			while (phnum-- && ph->p_type != PT_DYNAMIC)
// 				ph = (void*)((size_t)ph + phent);
// 			dynv = (void*)(base + ph->p_vaddr);
// 		}
// #endif
//
// 		for (i = 0; i < DYN_CNT; i++) dyn[i] = 0;
// 		for (i = 0; dynv[i]; i += 2) if (dynv[i] < DYN_CNT)
// 			dyn[dynv[i]] = dynv[i + 1];
//
// #if DL_FDPIC
// 		for (i = 0; i < DYN_CNT; i++) {
// 			if (i == DT_RELASZ || i == DT_RELSZ) continue;
// 			if (!dyn[i]) continue;
// 			for (j = 0; dyn[i] - segs[j].p_vaddr >= segs[j].p_memsz; j++);
// 			dyn[i] += segs[j].addr - segs[j].p_vaddr;
// 		}
// 		base = 0;
//
// 		const Sym* syms = (void*)dyn[DT_SYMTAB];
//
// 		rel = (void*)dyn[DT_RELA];
// 		rel_size = dyn[DT_RELASZ];
// 		for (; rel_size; rel += 3, rel_size -= 3 * sizeof(size_t)) {
// 			if (!IS_RELATIVE(rel[1], syms)) continue;
// 			for (j = 0; rel[0] - segs[j].p_vaddr >= segs[j].p_memsz; j++);
// 			size_t* rel_addr = (void*)
// 				(rel[0] + segs[j].addr - segs[j].p_vaddr);
// 			if (R_TYPE(rel[1]) == REL_FUNCDESC_VAL) {
// 				*rel_addr += segs[rel_addr[1]].addr
// 					- segs[rel_addr[1]].p_vaddr
// 					+ syms[R_SYM(rel[1])].st_value;
// 				rel_addr[1] = dyn[DT_PLTGOT];
// 			}
// 			else {
// 				size_t val = syms[R_SYM(rel[1])].st_value;
// 				for (j = 0; val - segs[j].p_vaddr >= segs[j].p_memsz; j++);
// 				*rel_addr = rel[2] + segs[j].addr - segs[j].p_vaddr + val;
// 			}
// 		}
// #else
// 		/* If the dynamic linker is invoked as a command, its load
// 		 * address is not available in the aux vector. Instead, compute
// 		 * the load address as the difference between &_DYNAMIC and the
// 		 * virtual address in the PT_DYNAMIC program header. */
// 		base = aux[AT_BASE];
// 		if (!base) {
// 			size_t phnum = aux[AT_PHNUM];
// 			size_t phentsize = aux[AT_PHENT];
// 			Phdr* ph =(Phdr*) (void*)aux[AT_PHDR];
// 			for (i = phnum; i--; ph = (Phdr*)(void*)((char*)ph + phentsize)) {
// 				if (ph->p_type == PT_DYNAMIC) {
// 					base = (size_t)dynv - ph->p_vaddr;
// 					break;
// 				}
// 			}
// 		}
//
// 		rel = (size_t*)(void*)(base + dyn[DT_REL]);
// 		rel_size = dyn[DT_RELSZ];
// 		for (; rel_size; rel += 2, rel_size -= 2 * sizeof(size_t)) {
// 			if (!IS_RELATIVE(rel[1], 0)) continue;
// 			size_t* rel_addr = (size_t*)(void*)(base + rel[0]);
// 			*rel_addr += base;
// 		}
//
// 		rel = (size_t*)(void*)(base + dyn[DT_RELA]);
// 		rel_size = dyn[DT_RELASZ];
// 		for (; rel_size; rel += 3, rel_size -= 3 * sizeof(size_t)) {
// 			if (!IS_RELATIVE(rel[1], 0)) continue;
// 			size_t* rel_addr = (size_t*)(void*)(base + rel[0]);
// 			*rel_addr = base + rel[2];
// 		}
//
// 		rel = (size_t*)(void*)(base + dyn[DT_RELR]);
// 		rel_size = dyn[DT_RELRSZ];
// 		size_t* relr_addr = 0;
// 		for (; rel_size; rel++, rel_size -= sizeof(size_t)) {
// 			if ((rel[0] & 1) == 0) {
// 				relr_addr = (size_t*)(void*)(base + rel[0]);
// 				*relr_addr++ += base;
// 			}
// 			else {
// 				for (size_t i = 0, bitmap = rel[0]; bitmap >>= 1; i++)
// 					if (bitmap & 1)
// 						relr_addr[i] += base;
// 				relr_addr += 8 * sizeof(size_t) - 1;
// 			}
// 		}
// #endif
//
// 		stage2_func dls2;
// 		GETFUNCSYM(&dls2, __dls2, base + dyn[DT_PLTGOT]);
// 		dls2((unsigned char*)(void*)base, sp);
//
// 	}
// };
