/*******************************************************************************
Copyright (c) 2021, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
	* Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	* Neither the name of the <organization> nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <stdint.h>

#include <sys/mman.h>

#include <pthread.h>

#include <bits/sysbits.h>
#include <bits/syscall.h>

#include "mutex_internal.h"

extern "C" {
    int pthread_mutexattr_init(pthread_mutexattr_t* matr) {
        uint32_t exp = 1;
        uint32_t des = 0;
        matr =(pthread_mutexattr_t*) mmap(0,sizeof(pthread_mutexattr_t),PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_32BIT, -1, 0);
        if((matr->pshared=sys::__internal::__init_mutex(exp,des))){
            return 0;
        }
        return -1;
    }

    int pthread_mutexattr_destroy(pthread_mutexattr_t* matr) {
        uint32_t exp = 0;
        uint32_t des = 1;
        uint32_t shared=matr->pshared;
        int ret = sys::__internal::__destroy_mutex(&shared,exp,des);
        munmap(matr,sizeof(pthread_mutexattr_t));
        return ret;
    }

    pthread_mutex_t __pthread_mutex_init() {
        pthread_mutex_t mtx;
        pthread_mutex_init(&mtx, nullptr);
        return mtx;
    };

    int pthread_mutex_init(pthread_mutex_t* __restrict__ mutex,
        const pthread_mutexattr_t* __restrict__ matr) {
        if (!mutex)
            return -1;
        return 0;
    }

    int pthread_mutex_destroy(pthread_mutex_t* mutex) {
        return 0;
    }

    int pthread_mutex_lock(pthread_mutex_t* mutex) {
        uint32_t exp = 0;
        uint32_t des = 1;
        return sys::__internal::__lock_mutex((uint32_t*)mutex,exp,des);
    }

    int pthread_mutex_trylock(pthread_mutex_t* mutex) {
        uint32_t exp = 0;
        uint32_t des = 1;
        return sys::__internal::__try_lock_mutex((uint32_t*)mutex,exp,des);
    }

    int pthread_mutex_unlock(pthread_mutex_t* mutex) {
        uint32_t exp = 1;
        uint32_t des = 0;
        return sys::__internal::__unlock_mutex((uint32_t*)mutex,exp,des);
    }
};

