/*******************************************************************************
Copyright (c) 2023, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <bits/syscall.h>
#include <bits/sysbits.h>
#include <sys/mman.h>

#pragma once

#define FUTEX_WAIT    0
#define FUTEX_WAKE    1

#ifdef __cplusplus

namespace sys {
    namespace __internal {

        inline uint32_t __init_mutex(uint32_t exp, uint32_t des) {
            uint32_t _mutex[2];
            _mutex[0] = (unsigned long)mmap(nullptr, sizeof(uint32_t),
                PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_SHARED| MAP_32BIT, -1, 0);
            *_mutex =uint32_t(1);

            return _mutex[0];
        };

        inline uint32_t __lock_mutex(uint32_t mutex[], uint32_t exp, uint32_t des) {
            if (!mutex)
                return -1;
            long s = -1;
            if (__atomic_compare_exchange(mutex, &exp, &des, false,
                __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST)) {
                s = syscall6(__NR_futex, (unsigned long)mutex, FUTEX_WAIT, 0, 0, 0, 0);
            }
            return s;
        };


        inline uint32_t __try_lock_mutex(uint32_t mutex[], uint32_t exp, uint32_t des) {
            if (!mutex)
                return -1;

            if (__atomic_compare_exchange(mutex, &exp, &des, false,
                __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST)) {
                long s = syscall6(__NR_futex, (unsigned long)mutex, FUTEX_WAKE, 1, 0, 0, 0);
                if (s == -1) {
                    return -1;
                }
            }
            return 0;
        };

        inline uint32_t __unlock_mutex(uint32_t mutex[], uint32_t exp, uint32_t des) {
            if (!mutex)
                return -1;
            if (__atomic_compare_exchange(mutex, &exp, &des, false,
                __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST)) {
                return syscall6(__NR_futex, (unsigned long)mutex, FUTEX_WAKE, 1, 0, 0, 0);
            }
            return -1;
        };

        inline uint32_t __destroy_mutex(uint32_t mutex[], uint32_t exp, uint32_t des) {
            __lock_mutex(mutex, exp, des);
            return munmap(mutex, sizeof(uint32_t));
        };
    };
};

#endif
