/*******************************************************************************
Copyright (c) 2021, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>

#include <bits/syscall.h>
#include <bits/sysbits.h>

#include "file.h"

extern "C" {
    
    int open(const char *pathname, int flags, mode_t mode){
        return sys::TOpen<int,mode_t>(pathname,flags,mode);
    }

    int close(int fd){
        return sys::TClose<int>(fd);
    }

    int creat(const char *pathname, mode_t mode){
        return sys::TOpen<int,mode_t>(pathname,O_WRONLY|O_CREAT|O_TRUNC,mode);
    }

    int openat(int dirfd, const char *pathname, int flags, mode_t mode){
        return sys::TOpenAt<int,mode_t>(dirfd,pathname,flags,mode);
    }

    FILE *fopen(const char * filename, const char * mode ){
        auto cflags = [](const char *fmode){
            int flags;
            if (strchr(fmode, '+')) flags = O_RDWR;
            else if (*fmode == 'r') flags = O_RDONLY;
            else flags = O_WRONLY;
            if (strchr(fmode, 'x')) flags |= O_EXCL;
            if (strchr(fmode, 'e')) flags |= O_CLOEXEC;
            if (*fmode != 'r') flags |= O_CREAT;
            if (*fmode == 'w') flags |= O_TRUNC;
            if (*fmode == 'a') flags |= O_APPEND;
            return flags;
        };

        FILE *file = new FILE;        
        try{           
            file->__x = open(filename, cflags(mode),0);
            return file;
        }catch(...){
            delete file;
            return nullptr;
        }
    }

    size_t fwrite(const void *ptr,size_t size,size_t count,FILE *stream){
        if(!stream)
            return -1;
        try{
            return write(stream->__x,ptr,(size*count));
        }catch(...){
            return -1;
        }
    }

    size_t fread (void * ptr, size_t size, size_t count, FILE * stream){
        if(!stream)
            return -1;
        try{
            return read(stream->__x,ptr,(size*count));
        }catch(...){
            return -1;
        }
    }
    
    void fclose(FILE *fd){
        if(close(fd->__x)){
            delete fd;
        }

    }
    
    int snprintf(char* s, size_t n, const char* format, ...) {
        return 0;
    }

    int vfprintf(FILE* stream, const char* format, va_list arg) {
        return 0;
    }

    int printf(const char* format, ...) {
        int n;
        va_list vl;
        va_start(vl, n);
        //for (int i = 0; i < n; i++){
        //    puts(va_arg(vl, char*));
        //}
        for (int i = 0; i < strlen(format); ++i) {
            if (format[i] != '%') {
                ++i;
                if (format[i]=='d') {
                    putchar(va_arg(vl,int));
                }
                if (format[i] == 's') {
                    puts(va_arg(vl, const char*));
                }

            }
            putchar(format[i]);
        }
        va_end(vl);

        return 0;
    }

    int putchar(int character) {
        return sys::TWrite<int, const void*>(stdout, &character,1);
    }

    int puts(const char* str) {
        return sys::TWrite<int, const void*>(stdout, str, strlen(str));
    }
};
