/*******************************************************************************
Copyright (c) 2023, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
	* Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	* Neither the name of the <organization> nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <pthread.h>
#include <config.h>

#include "mutex_internal.h"

#define SIGPT_SET \
	((sigset_t *)(const unsigned long [_NSIG/8/sizeof(long)]){ \
	[sizeof(long)==4] = 3UL<<(32*(sizeof(long)>4)) })

#define SIGSYNCCALL 34

#pragma once


#ifdef __cplusplus
namespace sys {
    class thread;
    namespace __internal {
        class thread{
        public:
            VISIBILTY_HIDDEN static thread& getins();
            VISIBILTY_HIDDEN sys::thread* self();
            VISIBILTY_HIDDEN void addins(sys::thread* thrd, size_t tid);
            VISIBILTY_HIDDEN void delins(sys::thread* thrd);
            VISIBILTY_HIDDEN void* getDesc(size_t tid);
            VISIBILTY_HIDDEN int getPid(size_t tid);
            VISIBILTY_HIDDEN void lock(volatile int* l);
            VISIBILTY_HIDDEN void unlock(volatile int* l);
        private:
            struct ins {
                int          tid;
                sys::thread* threadins;
                ins* nextins;
                ins();
                ~ins();
            };

            ins* _firstins;
            ins* _lastins;

            int *_mutex;

            thread();
            ~thread();

            friend void* ::pthread_self(void);
            friend int ::pthread_key_create(pthread_key_t* key, void (*destructor)(void*));
        };

        struct pthread_key_struct {
            uintptr_t seq;
            void (*destr) (void*);
        };

        extern struct pthread_key_struct __pthread_keys[128];

        struct pthread_key_data{
            /* Sequence number.  We use uintptr_t to not require padding on
               32- and 64-bit machines.  On 64-bit machines it helps to avoid
               wrapping, too.  */
            uintptr_t seq;

            /* Data pointer.  */
            void* data;
        };
        extern struct pthread_key_data specific_1stblock[32];
    };
};
#endif


