/*******************************************************************************
Copyright (c) 2022, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
	* Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	* Neither the name of the <organization> nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <string.h>
#include <unistd.h>

#include <sys/mman.h>

#include <bits/sysbits.h>
#include <bits/syscall.h>

#include "array.h"
#include "file.h"

extern "C" {
    int pipe(int fd[2]){
        return syscall1(__NR_pipe, (unsigned long)&fd);
    }
    
    pid_t fork(void){
        return syscall0(__NR_fork);
    }

	int chroot(const char *path){
		return syscall1(__NR_chroot,(unsigned long)path);
	}

	int chdir(const char *path){
		return syscall1(__NR_chdir,(unsigned long)path);
	}

	char *getcwd(char *buf, size_t size){
		return (char*)syscall2(__NR_getcwd,(unsigned long)buf,size);
	}

	char *getwd(char *buf){
		return getcwd(buf,strlen(buf));
	}

	int setuid(uid_t uid){
		return syscall1(__NR_setuid,uid);
	}
	
	uid_t getuid(void){
		return syscall0(__NR_getuid);
	}

	ssize_t write(int fd, const void *buf, size_t count){
		return sys::TWrite<int,const void*>(fd,buf,count);
	}

	ssize_t read(int fd, void *buf, size_t count){
		return sys::TRead<int,void*>(fd,buf,count);
	}

	int chmod(const char *pathname, mode_t mode){
        return syscall2(__NR_chmod,(unsigned long)pathname,mode);
    }
	
    int execve(const char *pathname, char *const argv[],
                  char *const envp[]){
        
        int a =1,aa=0,e=0,ee=0;
        
        char **argv_p;
        while(argv[a]){
            ++a;
        }
        argv_p = new char*[a-1];
        
        a=1;
        
        while(argv[aa]){
            argv_p[aa]=new char[strlen(argv[a])+1];
            memcpy(argv_p[aa],argv[a],strlen(argv[a]));
            ++aa; 
            ++a;
        }
        
        char **envp_p;
        while(envp[e]){
            ++e;
        }
        envp_p = new char*[e];
        
        while(envp[ee]){
            envp_p[ee]=new char[strlen(envp[ee])+1];
            memcpy(envp_p[ee],envp[ee],strlen(envp[ee]));
            ++ee;
        }

        return syscall3(__NR_execve,(unsigned long)pathname,
                        (unsigned long)argv_p,(unsigned long)envp_p);
    }
    
    int getpagesize(void){
        size_t n;
        char *p;
        int u;
        for (n = 1; n; n *= 2) {
            p = (char*)mmap(0, n * 2, PROT_NONE, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
            if (p == MAP_FAILED)
                return -1;
            u = munmap(p + n, n);
            munmap(p, n * 2);
            if (!u)
                return n;
        }
        return -1;
    }

    ssize_t readlink(const char* path, char* buf, size_t bufsize) {
        char dummy[1];
        if (!bufsize) {
            buf = dummy;
            bufsize = 1;
        }
#ifdef __NR_readlink
        int r = syscall3(__NR_readlink, (unsigned long)path, (unsigned long) buf, bufsize);
#else
        int r = syscall4(__NR_readlinkat, AT_FDCWD,(unsigned long) path, (unsigned long) buf, bufsize);
#endif
        if (buf == dummy && r > 0) 
            r = 0;
        return syscall_ret(r);
    }

    long sysconf(int name){
        return -1;
    }

    int isatty(int fd){
        return 0;
    }

    int sleep(int seconds){
        struct timespec treq;
        treq.tv_sec=seconds;
        treq.tv_nsec=0;
        return syscall4(__NR_clock_nanosleep,0,0,(long)&treq,0);
    }
}
