/*******************************************************************************
Copyright (c) 2018, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>

#include "systempp.h"

bool sys::__internal::systempp::Threaded = false;

extern "C" {
    
    typedef void initfunc_t(void);
    extern initfunc_t* __init_array_start[], * __init_array_end[];

    static void handle_init_array(void)
    {
        for (initfunc_t** p = __init_array_start; p != __init_array_end; p++)
            (*p)();
    }

    static int libc_start_main_stage2(int (*main)(int,char **,char **), int argc, char **argv){        
        char **envp = argv+argc+1;
        /* Pass control to the application */
        return main(argc, argv, envp);
    }
    
    __attribute__ ((__visibility__ ("default"))) 
    int __libc_start_main(int (*main)(int,char **,char **), int argc, char **argv,
                                void (*init_dummy)(), void(*fini_dummy)(), void(*ldso_dummy)()){
        /* Barrier against hoisting application code or anything using ssp
         * or thread pointer prior to its initialization above. */
        lsm2_fn *stage2 = libc_start_main_stage2;
        __asm__ ( "" : "+r"(stage2) : : "memory" );
        exit(stage2(main, argc, argv));
        return 0;
    }

    __attribute__((weak, noreturn)) void __stack_chk_fail(void){
        puts("Stack overflow detected! Aborting program.");
        abort();
    }
};
